import { TemplateResult } from 'lit';
import { ComponentBase } from '../../base/component-base/component.base';
import { BaseSizeTypes, SizeTypes } from '../../types/sizes.type';
import { ButtonColors, ButtonExposeVar, ButtonIconSides, ButtonSizes, ButtonTypes } from './types/button.config.types';
import '../icon/icon';
import { IconSrcTypes } from '../icon/types/icon.types';
import '../spinner/spinner';
import { CxButtonPressed } from './types/button.types';
import { CxButtonName } from './types/button.name';
import { ColorTypes } from '../../types/colors.type';
export declare class Button extends ComponentBase<CXButton.Props> {
    config: CXButton.Props['set'];
    static styles: import("lit").CSSResult;
    connectedCallback(): void;
    render(): TemplateResult;
    private setSpinnerConfig;
    private renderIconLeft;
    private renderContent;
    private renderIconRight;
    updated(changedPros: Map<PropertyKey, unknown>): void;
    onLog(config: {
        text: string;
    }): void;
    private pressed;
}
declare global {
    namespace CXButton {
        type Ref = Button;
        type Var = {
            width?: SizeTypes;
            height?: SizeTypes;
            textColor?: ColorTypes;
            textActiveColor?: ColorTypes;
            textDisabledColor?: ColorTypes;
            backgroundColor?: ColorTypes;
            disabledColor?: ColorTypes;
            hoverColor?: ColorTypes;
            textHoverColor?: ColorTypes;
            activeColor?: ColorTypes;
            borderColor?: ColorTypes;
            borderDisabledColor?: ColorTypes;
            borderRadius?: BaseSizeTypes;
            borderWidth?: SizeTypes;
            outlineColor?: ColorTypes;
            outlineWidth?: SizeTypes;
            paddingLeft?: SizeTypes;
            paddingRight?: SizeTypes;
            paddingTop?: SizeTypes;
            paddingBottom?: SizeTypes;
            boxShadow?: ColorTypes;
            fontSize?: SizeTypes;
        };
        type Set = {
            disabled?: boolean;
            type?: ButtonTypes;
            color?: ButtonColors;
            size?: ButtonSizes;
            iconSide?: ButtonIconSides;
            iconOnly?: boolean;
            iconSrc?: IconSrcTypes;
            text?: string | number;
            loading?: boolean;
        };
        type Fix = Required<{
            [K in keyof Set]: (value: Set[K]) => Fix;
        }> & {
            exec: () => void;
        };
        type Props = {
            var: Pick<Var, ButtonExposeVar>;
            set: Set;
            fix: Fix;
            make: Var;
        };
        type Details = {
            [CxButtonPressed]: {
                event: string;
            };
        };
        type Events = {
            [CxButtonPressed]: (detail: Pressed) => void;
        };
        type Pressed = CustomEvent<Details[typeof CxButtonPressed]>;
    }
    interface HTMLElementTagNameMap {
        [CxButtonName]: CXButton.Ref;
    }
}
