var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { css, html } from 'lit';
import { customElement } from 'lit/decorators.js';
import { ComponentBase } from '../../base/component-base/component.base';
import '../icon/icon';
import '../spinner/spinner';
import ButtonFactory from './factory/factories/button.factory';
import { CxButtonPressed } from './types/button.types';
import { CxButtonName } from './types/button.name';
let Button = class Button extends ComponentBase {
    constructor() {
        super(...arguments);
        this.config = {
            disabled: false,
            type: 'primary',
            color: 'primary',
            size: 'medium',
            iconSrc: undefined,
            iconSide: 'prefix',
            iconOnly: false,
            text: '',
            loading: false,
        };
    }
    connectedCallback() {
        super.connectedCallback();
        if (this.set)
            this.cacheConfig(this.set);
        if (this.config)
            this.exec();
    }
    render() {
        return html `
      <style></style>
      <button
        icon-only="${this.set.iconOnly}"
        class="cx-button-container"
        @click="${this.pressed}"
        data-disabled="${this.set.disabled}"
        data-loading="${this.set.loading}">
        ${this.renderIconLeft()} ${this.renderContent()} ${this.renderIconRight()}
      </button>
    `;
    }
    setSpinnerConfig() {
        const spinnerSet = {
            color: this.set?.color,
            size: this.set?.size,
        };
        if (this.set?.type === 'primary') {
            spinnerSet.color = 'white';
        }
        else if (this.set?.color === 'primary' &&
            (this.set?.type === 'tertiary' || this.set.type === 'secondary-outline')) {
            spinnerSet.color = 'gray';
        }
        return spinnerSet;
    }
    renderIconLeft() {
        if (this.set?.loading && this.set?.iconOnly) {
            const spinnerSet = this.setSpinnerConfig();
            return html `<cx-spinner .set="${spinnerSet}"></cx-spinner>`;
        }
        if (this.set?.iconSrc && !this.set?.loading) {
            return html ` <cx-icon
        prefix="${this.set?.iconSide === 'prefix'}"
        icon-only="${this.set.iconOnly}"
        .set="${{
                src: this.set?.iconSrc,
                color: this.var?.textColor,
            }}"></cx-icon>`;
        }
    }
    renderContent() {
        if (this.set?.iconOnly)
            return;
        if (this.set?.loading) {
            const spinnerSet = this.setSpinnerConfig();
            return html `<cx-spinner .set="${spinnerSet}"></cx-spinner>`;
        }
        return html `<slot></slot>${this.set?.text}`;
    }
    renderIconRight() {
        if (this.set?.iconSrc && !this.set?.iconOnly && !this.set?.loading) {
            return html ` <cx-icon
        suffix="${this.set?.iconSide === 'suffix'}"
        .set="${{
                src: this.set?.iconSrc,
                color: this.var?.textColor,
            }}"></cx-icon>`;
        }
    }
    // life cycle
    updated(changedPros) {
        // 📌 offsetParent === null mean element is not visible from dom
        if (this.offsetParent === null)
            return;
        // 📌changedPros.has('set') will execute for init variables for 1st time and when "set" change
        if (changedPros.has('set')) {
            this.cacheVariables({ ...ButtonFactory.getCSSVariables(this.set), ...this.var });
            this.setHostVariables();
        }
        super.update(changedPros);
    }
    // Methods
    // FIXME: remove this method when everything is done!
    onLog(config) {
        console.log('Log:', config.text);
    }
    // Events
    pressed() {
        // test
        this.setCustomEvent(CxButtonPressed, {
            event: CxButtonPressed,
        });
    }
};
Button.styles = css `
    :host {
      /* default variable will not change */
      --fontSize: var(--size-16);
      --paddingLeft: var(--size-16);
      --paddingRight: var(--size-16);
      --paddingTop: var(--size-10);
      --paddingBottom: var(--size-10);
      --borderRadius: var(--base-size-8);
      --outlineWidth: var(--size-3);

      display: inline-block;
    }
    .cx-button-container {
      font-family: inherit;
      min-width: var(--width);
      font-size: var(--fontSize);
      height: var(--height);
      color: var(--textColor);
      background-color: var(--backgroundColor);
      padding-left: var(--paddingLeft);
      padding-right: var(--paddingRight);
      padding-top: var(--paddingTop);
      padding-bottom: var(--paddingBottom);
      border-radius: var(--borderRadius);
      border-style: solid;
      cursor: pointer;
      user-select: none;
      border-width: var(--borderWidth);
      border-color: var(--borderColor);
      transition: background-color 0.125s, outline-width 0.25s, outline-color 0.25s,
        box-shadow 0.25s;
      box-sizing: border-box;
      display: inline-flex;
      align-items: center;
      justify-content: center;
    }

    .cx-button-container[icon-only='true'] {
      min-width: var(--height);
      padding-left: var(--size-0);
      padding-right: var(--size-0);
      padding-top: var(--size-0);
      padding-bottom: var(--size-0);
    }

    .cx-button-container[data-disabled='true'] {
      background-color: var(--disabledColor);
      border-color: var(--borderDisabledColor);
      cursor: default;
      outline-width: var(--size-0);
      pointer-events: none;
      color: var(--textDisabledColor);
    }

    .cx-button-container[data-loading='true'] {
      pointer-events: none;
    }

    .cx-button-container:hover {
      background-color: var(--hoverColor);
      color: var(--textHoverColor);
      box-shadow: 0 0 var(--size-10) 0 var(--boxShadow);
    }

    .cx-button-container:active {
      background-color: var(--activeColor);
      color: var(--textActiveColor);
    }

    .cx-button-container:focus {
      outline-style: solid;
      outline-width: var(--outlineWidth);
      outline-color: var(--outlineColor);
    }
    cx-icon[prefix='true'] {
      margin-right: var(--size-10);
    }
    cx-icon[suffix='true'] {
      margin-left: var(--size-10);
    }

    cx-icon[prefix='true'],
    cx-icon[suffix='true'] {
      visibility: visible !important;
    }

    cx-icon[prefix='false'],
    cx-icon[suffix='false'] {
      visibility: hidden;
    }

    cx-icon[icon-only='true'] {
      margin-right: var(--size-0);
      visibility: visible !important;
    }
  `;
Button = __decorate([
    customElement(CxButtonName)
], Button);
export { Button };
//# sourceMappingURL=button.js.map