import { iconStyleMapper } from '../../styles-mapper/abbr-styles-mapper';
import { ScopedIcon } from '../scoped/ScopedIcon';
import { UIScopedStyles } from '../scoped/UIScoped';
export class IconToggleAttribute {
    constructor(box, attr, value) {
        this.box = box;
        this.attr = attr;
        this.value = value;
        this.toggleEvent = () => {
            this.box.iconToggled = !this.box.iconToggled;
            this.checkToggle();
        };
    }
    init() {
        if (this.isUiCacheValueSet())
            return;
        UIScopedStyles.setStylesheet();
        this.setValueCache();
        this.setCSSRule();
        this.setToggleEvent();
        this.box.removeAttribute(this.attr);
        this.box.valueCache?.set(this.attr, this.value);
        if (this.value !== 'none') {
            this.checkToggle();
        }
    }
    isUiCacheValueSet() {
        return this.box.valueCache?.get(this.attr) === this.value;
    }
    setToggleEvent() {
        if (this.box.iconToggled === undefined) {
            this.box.iconToggled = false;
            this.box.addEventListener('click', this.toggleEvent);
        }
    }
    checkToggle() {
        if (this.box.iconToggled) {
            this.box.setAttribute(this.attr, this.box.valueCache?.get(this.attr));
        }
        else {
            this.box.removeAttribute(this.attr);
        }
    }
    setValueCache() {
        if (!this.box.valueCache) {
            this.box.valueCache = new Map();
        }
    }
    setCSSRule() {
        if (this.value === 'none')
            return;
        const prefixSuffix = this.attr.split('-')[1] === 'prefix' ? '::before' : '::after';
        ScopedIcon.scope('prop', this.createStyleText(prefixSuffix), this.attr, this.box);
    }
    createStyleText(prefixSuffix, iconState) {
        const [size, source, color] = this.value.split(' ');
        if (size === 'none') {
            return `${iconState || ''}${prefixSuffix}{
        content: none;
      }
      `;
        }
        else {
            const [iconSize, iconSrc, iconColor] = iconStyleMapper.get(this.attr);
            const sourceWithImportant = source?.endsWith('!') ? '!important' : '';
            const sizeWithImportant = size?.endsWith('!') ? '!important' : '';
            const colorWithImportant = color?.endsWith('!') ? '!important' : '';
            return `${iconState || ''}${prefixSuffix}{
        content: '\uE800';
        ${iconSrc}: ${source.replace('!', '')}${sourceWithImportant};
        ${iconSize}: var(--size-${size.replace('!', '')})${sizeWithImportant};
        ${iconColor}: var(--${color.replace('!', '')})${colorWithImportant};
      }
      `;
        }
    }
}
//# sourceMappingURL=IconToggle.attribute.js.map