import { stylesMapper } from '../../styles-mapper/styles-mapper';
import { ScopedStyle } from '../scoped/ScopedStyles';
import { UIScopedStyles } from '../scoped/UIScoped';
export class UIToggleAttribute {
    constructor(attr, box, value) {
        this.attr = attr;
        this.box = box;
        this.value = value;
    }
    init() {
        UIScopedStyles.setStylesheet();
        const styles = this.getStyles();
        this.setUiStyle(styles);
        this.box.uiName = this.getUiName(styles);
        this.setToggleEvent();
        this.setUiAttrs();
    }
    setUiAttrs() {
        if (this.box.getAttribute(`_${this.attr}`)) {
            this.box.setAttribute(`_${this.attr}`, this.box.uiName);
        }
        this.box.removeAttribute(`${this.attr}`);
    }
    setToggleEvent() {
        if (this.box?.uiToggled === undefined) {
            this.box.uiToggled = false;
            this.box.addEventListener('click', () => {
                this.box.uiToggled = !this.box.uiToggled;
                if (this.box.uiToggled) {
                    this.box.setAttribute(`_${this.attr}`, this.box.uiName);
                }
                else {
                    this.box.removeAttribute(`_${this.attr}`);
                }
            });
        }
    }
    getStyles() {
        return this.value.split(',').map((style) => style.trim());
    }
    getUiAttrs(value) {
        return value.split(':').map((s) => s.trim());
    }
    getUiStyleText(uiStyle) {
        return uiStyle
            .split(' ')
            .filter(Boolean)
            .map((s) => {
            const styleProp = stylesMapper.get(`c-box[${s.replace('!', '').trim()}]`);
            return styleProp ? `${styleProp}${s?.endsWith('!') ? '!important' : ''};` : '';
        })
            .join('');
    }
    setUiStyle(styles) {
        for (const style of styles) {
            const [uiName, uiStyle] = this.getUiAttrs(style);
            if (uiName && uiStyle) {
                const styleText = this.getUiStyleText(uiStyle);
                ScopedStyle.scope('style', styleText, this.box, uiName, this.attr);
            }
        }
    }
    getUiName(styles) {
        return styles.map((s) => s.split(':')[0].trim()).join(' ');
    }
}
//# sourceMappingURL=UI-Toggle.attribute.js.map