var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { css, html } from 'lit';
import { customElement, property } from 'lit/decorators.js';
import { ComponentBase } from '../../base/component-base/component.base';
import '../button/button';
import { createRef, ref } from 'lit/directives/ref.js';
import './single-calendar/single-calendar';
import { convertDateToArrayNumber, } from '../../helpers/functions/date/date-methods';
import { mutableElement } from '../../helpers/functions/observe-element/mutable-element';
import { CxCalendarName } from './types/calendar.name';
import './month-calendar/month-calendar';
import './year-calendar/year-calendar';
import { DisplayState } from './states/DisplayState';
// export const onPressed = 'pressed';
let Calendar = class Calendar extends ComponentBase {
    constructor() {
        super(...arguments);
        this.config = {
            date: new Date(),
            display: '1-calendar',
            min: undefined,
            max: undefined,
            initValue: true,
            multiSelect: false,
            dateRange: false,
            dateValue: undefined,
            rangeValue: undefined,
        };
        // 📌 0 = previous month
        // 📌 -304 = current month
        // 📌 -608 = next month
        this.currentTranslateValue = -304;
        this.buttonLeftSet = {
            iconOnly: true,
            iconSrc: 'arrow-left-line',
            type: 'secondary',
            size: 'small',
        };
        this.buttonVar = {
            borderRadius: 'base-size-half',
        };
        this.buttonRightSet = {
            iconOnly: true,
            iconSrc: 'arrow-right-line',
            type: 'secondary',
            size: 'small',
        };
        this.calendarMonitorRef = createRef();
        this.displayState = 'date';
        this.monthDisplayRef = createRef();
        this.yearDisplayRef = createRef();
        this.selectDate = (e) => {
            if (this.set.dateRange) {
                const { endDate, startDate } = e.detail;
                this.setCustomEvent('select-date', {
                    endDate,
                    startDate,
                });
            }
            else {
                const { date } = e.detail;
                this.setCustomEvent('select-date', {
                    date,
                });
            }
        };
    }
    render() {
        return html `
      <style>
        :host {
          /* 📌default = current month */
          --translate: ${this.currentTranslateValue}px;
          --display-calendar: ${this.setDisplayCalendar()}px;
        }

        .calendar-monitor {
          translate: ${this.displayState === 'date' ? 'var(--translate)' : '0'};
        }
      </style>

      <!-- 📌 Calendar Display -->
      <div class="calendar-group">
        <cx-button
          class="handle-month-previous"
          @click="${this.goPrev}"
          .var="${this.buttonVar}"
          .set="${this.buttonLeftSet}"></cx-button>
        <cx-button
          class="handle-month-next"
          @click="${this.goNext}"
          .var="${this.buttonVar}"
          .set="${this.buttonRightSet}"></cx-button>
        <div class="calendar-monitor" ${ref(this.calendarMonitorRef)}>${this.renderCalendar()}</div>
      </div>
    `;
    }
    renderCalendar() {
        switch (this.displayState) {
            case 'date':
                return html `
          ${this.calendarGroup.map((calendar) => html ` <cx-single-calendar
                @select-date="${this.selectDate}"
                @calendar-display="${(e) => {
                    this.goToMonthCalendar(e.detail.date);
                }}"
                .set="${{ calendar, daterange: this.set.dateRange }}">
              </cx-single-calendar>`)}
        `;
            case 'month':
                return html `<cx-month-calendar ${ref(this.monthDisplayRef)}></cx-month-calendar>`;
            case 'year':
                return html `<cx-year-calendar ${ref(this.yearDisplayRef)}></cx-year-calendar>`;
            default:
                break;
        }
    }
    goToMonthCalendar(date) {
        this.setDisplayState('month');
        requestAnimationFrame(() => {
            if (!this.monthDisplayRef.value)
                return;
            this.monthDisplayRef.value.date = date;
            this.monthDisplayRef.value.calendar = this;
        });
    }
    setDisplayState(display) {
        this.displayState = display;
    }
    firstUpdated() {
        this.observeCalendarMonitor();
        this.updateSelectedDates();
    }
    updateSelectedDates() {
        const { initValue, dateValue, rangeValue, dateRange: daterange } = this.set;
        if (initValue && (dateValue || rangeValue)) {
            const calendarMonitor = this.calendarMonitorRef.value;
            const updateAttribute = (attributeName, date) => {
                const dateArray = convertDateToArrayNumber(date);
                if (calendarMonitor && dateArray) {
                    calendarMonitor.setAttribute(attributeName, dateArray.join('-'));
                }
            };
            if (daterange) {
                updateAttribute('startdate-selected', rangeValue?.startDate);
                updateAttribute('enddate-selected', rangeValue?.endDate);
                updateAttribute('latest-date-hover', rangeValue?.endDate);
            }
            else {
                updateAttribute('single-selected', dateValue.date);
            }
        }
    }
    observeCalendarMonitor() {
        mutableElement(this.calendarMonitorRef.value, 'attributes', (m) => {
            const singleCalendars = m.target
                .children;
            for (const singleCalendar of singleCalendars) {
                singleCalendar.updateSelected();
            }
        });
    }
    setDisplayCalendar() {
        switch (this.set.display) {
            default:
            case '1-calendar':
                return 304;
            case '2-calendars':
                return 608;
        }
    }
    connectedCallback() {
        super.connectedCallback();
        if (this.set)
            this.cacheConfig(this.set);
        if (this.config)
            this.exec();
        this.calendarDisplayMethods = new DisplayState(this);
    }
    willUpdate(changedProps) {
        if (this.set.rangeValue?.startDate) {
            this.dateNavigator ||= this.set.rangeValue?.startDate;
        }
        this.calendarDisplayMethods?.generate();
        super.willUpdate(changedProps);
    }
    goPrev() {
        this.calendarDisplayMethods?.goPrev();
    }
    goNext() {
        this.calendarDisplayMethods?.goNext();
    }
};
Calendar.styles = css `
    :host {
      display: inline-block;
    }
    .calendar-group {
      max-width: var(--display-calendar);
      overflow: hidden;
      background-color: var(--white);
      padding: 28px 12px 20px;
      border-radius: var(--base-size-16);
      position: relative;
    }

    .calendar-monitor {
      /* 0 is previous month */
      /* -304 is current month */
      /* -608 is next month */
      display: flex;
      transition: translate 0.25s ease-out;
      transition-timing-function: cubic-bezier(0.1, 0.2, 0.2, 1);
      /* 📌improve ux speed */
    }

    .handle-month-previous,
    .handle-month-next {
      position: absolute;
      top: var(--size-14);
      z-index: 1;
    }
    .handle-month-previous {
      left: var(--size-14);
    }
    .handle-month-next {
      right: var(--size-14);
    }
  `;
__decorate([
    property({ type: Date }),
    __metadata("design:type", Date)
], Calendar.prototype, "dateNavigator", void 0);
__decorate([
    property({ type: String }),
    __metadata("design:type", String)
], Calendar.prototype, "displayState", void 0);
Calendar = __decorate([
    customElement(CxCalendarName)
], Calendar);
export { Calendar };
//# sourceMappingURL=calendar.js.map