import { convertToDate } from '../../../../helpers/functions/date/date-methods';
export class CalendarBaseMethod {
    // utilities
    constructor(signleCalendar) {
        this.signleCalendar = signleCalendar;
    }
    getDateFromString(dateStr) {
        const [year, month, date] = dateStr.split('-').map(Number);
        return convertToDate(year, month, date);
    }
    getUpdatedStartEndAttributes() {
        return [
            this.getCalendarMonitorAttr('startdate-selected'),
            this.getCalendarMonitorAttr('enddate-selected'),
        ];
    }
    setOldDateRange() {
        const [startdate, enddate] = this.getUpdatedStartEndAttributes();
        this.signleCalendar.parentElement?.setAttribute('old-startdate', startdate);
        this.signleCalendar.parentElement?.setAttribute('old-enddate', enddate);
    }
    removeStartEndDateAttribute(type) {
        this.signleCalendar.parentElement?.removeAttribute(`${type}-selected`);
    }
    setSingleSelectAttribute(year, month, date) {
        this.signleCalendar.parentElement?.setAttribute('single-selected', `${year}-${month}-${date}`);
    }
    setStartEndDateAttribute(type, year, month, date) {
        this.signleCalendar.parentElement?.setAttribute(`${type}-selected`, `${year}-${month}-${date}`);
    }
    getDateSelected(e) {
        const dateDOMSelected = e.target.closest('.date');
        if (!dateDOMSelected)
            throw 'Wrong date selection';
        return dateDOMSelected.dataset.date.split('-');
    }
    removeDateBetweenClass(shadowRootSheet) {
        if (shadowRootSheet?.cssRules.length !== 0)
            shadowRootSheet?.deleteRule(0);
    }
    dateConverted(day) {
        if (!this.signleCalendar.set.calendar)
            return;
        return convertToDate(this.signleCalendar.set.calendar.year, this.signleCalendar.set.calendar.month, day);
    }
    getCalendarMonitorAttr(attributeName) {
        return this.signleCalendar.parentElement?.getAttribute(attributeName);
    }
    removeSelection() {
        if (this.signleCalendar.dateSelectedDOM) {
            this.signleCalendar.dateSelectedDOM.classList.remove('selected');
        }
    }
}
//# sourceMappingURL=calendar-base.js.map