import { generateYearGroup, getCalendarDetail, getNextMonth, getPreviousMonth, } from '../../../helpers/functions/date/date-methods';
export class DisplayState {
    constructor(calendar) {
        this.calendar = calendar;
    }
    generate() {
        switch (this.calendar.displayState) {
            case 'date':
                return new DateState(this.calendar).generateCalendar();
            default:
                break;
        }
    }
    goPrev() {
        switch (this.calendar.displayState) {
            case 'date':
                return new DateState(this.calendar).goPrevMonth();
            case 'month':
                return new MonthState(this.calendar).goPrevYear();
            case 'year':
                return new YearState(this.calendar).goPrevYearGroup();
            default:
                break;
        }
    }
    goNext() {
        switch (this.calendar.displayState) {
            case 'date':
                return new DateState(this.calendar).goNextMonth();
            case 'month':
                return new MonthState(this.calendar).goNextYear();
            case 'year':
                return new YearState(this.calendar).goNextYearGroup();
            default:
                break;
        }
    }
}
export class DateState {
    constructor(calendar) {
        this.calendar = calendar;
    }
    generateCalendar() {
        const currentMonth = this.calendar.dateNavigator || this.calendar.set.date;
        const previousMonth = getPreviousMonth(currentMonth);
        const nextMonth = getNextMonth(currentMonth);
        switch (this.calendar.set.display) {
            case '1-calendar':
                this.calendar.calendarGroup = [
                    getCalendarDetail({
                        date: previousMonth,
                        min: this.calendar.set.min,
                        max: this.calendar.set.max,
                    }),
                    getCalendarDetail({
                        date: currentMonth,
                        min: this.calendar.set.min,
                        max: this.calendar.set.max,
                    }),
                    getCalendarDetail({
                        date: nextMonth,
                        min: this.calendar.set.min,
                        max: this.calendar.set.max,
                    }),
                ];
                break;
            case '2-calendars':
                this.calendar.calendarGroup = [
                    getCalendarDetail({
                        date: previousMonth,
                        min: this.calendar.set.min,
                        max: this.calendar.set.max,
                    }),
                    getCalendarDetail({
                        date: currentMonth,
                        min: this.calendar.set.min,
                        max: this.calendar.set.max,
                    }),
                    getCalendarDetail({
                        date: nextMonth,
                        min: this.calendar.set.min,
                        max: this.calendar.set.max,
                    }),
                    getCalendarDetail({
                        date: getNextMonth(nextMonth),
                        min: this.calendar.set.min,
                        max: this.calendar.set.max,
                    }),
                ];
                break;
        }
    }
    goPrevMonth() {
        this.translateMonth('previous');
        const timer = setTimeout(() => {
            const singleCalendar = this.createSingleCalendar('previous', this.calendar.calendarMonitorRef.value?.firstElementChild);
            this.removeUnusedCalendar(this.calendar.calendarMonitorRef.value?.lastElementChild);
            this.appendNewCalendar('previous', singleCalendar);
            this.setTransitionCalendar('previous');
            clearTimeout(timer);
        }, 250);
    }
    setTransitionCalendar(type) {
        const calendarMonitor = this.calendar.calendarMonitorRef.value;
        if (!calendarMonitor)
            return;
        calendarMonitor.style.transition = 'none';
        if (type === 'previous') {
            this.calendar.currentTranslateValue -= 304;
        }
        else if (type === 'next') {
            this.calendar.currentTranslateValue += 304;
        }
        this.calendar.style.setProperty('--translate', `${this.calendar.currentTranslateValue}px`);
    }
    appendNewCalendar(type, singleCalendar) {
        const calendarMonitor = this.calendar.calendarMonitorRef.value;
        if (!calendarMonitor)
            return;
        if (type === 'previous') {
            calendarMonitor.insertBefore(singleCalendar, calendarMonitor.firstElementChild);
        }
        else {
            calendarMonitor.appendChild(singleCalendar);
        }
    }
    translateMonth(direction) {
        this.calendar.calendarMonitorRef.value.style.transition = '0.25s ease-out';
        this.calendar.currentTranslateValue += direction === 'previous' ? 304 : -304;
        this.calendar.style.setProperty('--translate', `${this.calendar.currentTranslateValue}px`);
    }
    removeUnusedCalendar(focussedCalendar) {
        if (focussedCalendar) {
            this.calendar.calendarMonitorRef.value?.removeChild(focussedCalendar);
        }
    }
    createSingleCalendar(type, focusedCalendar) {
        const previousMonthFromMonthVisibled = type === 'previous'
            ? getPreviousMonth(focusedCalendar.set.calendar?.firstDateOfMonth)
            : getNextMonth(focusedCalendar.set.calendar?.firstDateOfMonth);
        const generatedMonth = getCalendarDetail({
            date: previousMonthFromMonthVisibled,
            min: this.calendar.set.min,
            max: this.calendar.set.max,
        });
        const singleCalendar = document.createElement('cx-single-calendar');
        singleCalendar.fix().calendar(generatedMonth).daterange(this.calendar.set.dateRange).exec();
        singleCalendar.addEventListener('select-date', this.calendar.selectDate);
        singleCalendar.addEventListener('calendar-display', (e) => {
            this.calendar.goToMonthCalendar(e.detail.date);
        });
        return singleCalendar;
    }
    goNextMonth() {
        this.translateMonth('next');
        const timer = setTimeout(() => {
            const singleCalendar = this.createSingleCalendar('next', this.calendar.calendarMonitorRef.value.lastElementChild);
            this.removeUnusedCalendar(this.calendar.calendarMonitorRef.value?.firstElementChild);
            this.appendNewCalendar('next', singleCalendar);
            this.setTransitionCalendar('next');
            clearTimeout(timer);
        }, 250);
    }
}
export class MonthState {
    constructor(calendar) {
        this.calendar = calendar;
        this.mothDisplayRef = calendar.monthDisplayRef.value;
    }
    goPrevYear() {
        this.mothDisplayRef?.calcYear(this.mothDisplayRef.date, 'prev');
    }
    goNextYear() {
        this.mothDisplayRef?.calcYear(this.mothDisplayRef.date, 'next');
    }
}
export class YearState {
    constructor(calendar) {
        this.calendar = calendar;
        this.yearDisplayRef = calendar.yearDisplayRef.value;
    }
    goPrevYearGroup() {
        if (!this.yearDisplayRef)
            return;
        const targetYear = new Date(this.yearDisplayRef.years[0], 1);
        this.yearDisplayRef.years = generateYearGroup(targetYear, 11);
    }
    goNextYearGroup() {
        if (!this.yearDisplayRef)
            return;
        const targetYear = this.yearDisplayRef.years[this.yearDisplayRef.years.length - 1] + 1;
        const newDate = new Date(targetYear, 1);
        newDate.setFullYear(newDate.getFullYear() + 11);
        this.yearDisplayRef.years = generateYearGroup(newDate, 11);
    }
}
//# sourceMappingURL=DisplayState.js.map