var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { css, html } from 'lit';
import { customElement } from 'lit/decorators.js';
import { createRef, ref } from 'lit/directives/ref.js';
import { ComponentBase } from '../../base/component-base/component.base';
import { ModalSingleton } from '../modal/singleton/modal.singleton';
import { DialogState } from '../modal/state/dialog.state';
import '../transition/transition';
import { CxDialogName } from './types/dialog.name';
import { CxDialogOnClosed } from './types/dialog.types';
let Dialog = class Dialog extends ComponentBase {
    constructor() {
        super(...arguments);
        this.config = {
            disabledBackdrop: false,
            transition: {
                start: {
                    name: 'scale-up',
                    delay: '0',
                    duration: '0.25s',
                    timing: 'ease',
                },
                end: {
                    name: 'scale-down',
                    delay: '0',
                    duration: '0.25s',
                    timing: 'ease',
                },
            },
        };
        this.dialogRef = createRef();
        this.transitionRef = createRef();
    }
    connectedCallback() {
        super.connectedCallback();
        if (this.set)
            this.cacheConfig(this.set);
        if (this.config)
            this.exec();
    }
    render() {
        return this.slot
            ? html `
          <style></style>
          <cx-transition ${ref(this.transitionRef)} .set="${this.set.transition}">
            <div class="dialog" ${ref(this.dialogRef)}>
              <slot></slot>
            </div>
          </cx-transition>
        `
            : undefined;
    }
    // 📌Call method from Modal Component
    executeTransition(when) {
        if (!this.transitionRef?.value)
            return;
        this.transitionRef.value.setTransition(when);
    }
    // 📌this method *open/close only use for local dialog
    open() {
        this.setSlotName();
        this.openLocalDialog();
    }
    setSlotName(globalSlotName) {
        if (this.slot)
            return;
        this.slot = globalSlotName || DialogState.LOCAL_DIALO_SLOT;
        this.requestUpdate();
    }
    openLocalDialog() {
        ModalSingleton.modalRef.append(this);
        ModalSingleton.modalRef.openDialog(DialogState.LOCAL_DIALO_SLOT);
    }
    close() {
        ModalSingleton.modalRef?.closeDialog();
    }
    onClosed() {
        this.setCustomEvent(CxDialogOnClosed, {
            event: CxDialogOnClosed,
        });
    }
};
Dialog.styles = css `
    .dialog {
      display: inline-block;
      background-color: var(--white);
      padding: var(--base-size-20);
      border-radius: var(--base-size-8);
    }
  `;
Dialog = __decorate([
    customElement(CxDialogName)
], Dialog);
export { Dialog };
//# sourceMappingURL=dialog.js.map