import { mutableElementManyTypes } from '../../../helpers/functions/observe-element/mutable-element';
import { Modal } from '../modal';
import { ModalSingleton } from '../singleton/modal.singleton';
export class DialogState {
    constructor() {
        this.open = (detail) => {
            this.setSlotName(detail.slotName);
            this.setDialogSlot(detail.slotRef);
            if (this.dialogSlot?.value?.name === DialogState.DIALOG_SLOT_DEFAULT) {
                this.startTransition();
                this.toggleBackdrop('enabled');
                this.SetOpacity('1');
                this.setModalSlot();
            }
            else {
                // 📌close when call new dialog while current dialog still appear
                this.close(false);
                // 📌after close then it will open new dialog immediately
                this.openChildDialog();
            }
            return this.getAsyncDialogRef();
        };
        this.close = (shouldTriggerOnClosed = true) => {
            this.removeEscapeKeyupEvent();
            this.SetOpacity('0');
            this.endTransition();
            this.closeDialogWhenEnd();
            this.triggerOnClosedWhenTimeup(shouldTriggerOnClosed);
        };
        this.closeBackdrop = () => {
            if (ModalSingleton.modalRef.set.disabledBackdrop || this.dialogRef?.set.disabledBackdrop)
                return;
            this.close();
        };
        // 📌need to use arrow function becoz this function is called from another scope
        this.setModalSlot = (slotName) => {
            if (this.dialogSlot?.value) {
                this.dialogSlot.value.name = slotName || this.slotName;
            }
        };
        this.disabledBackdropByEscape = (e) => {
            if (e.key !== 'Escape')
                return;
            this.close();
        };
    }
    setSlotName(slotName) {
        this.slotName = slotName;
    }
    setDialogSlot(slotRef) {
        if (!this.dialogSlot)
            this.dialogSlot = slotRef;
    }
    getAsyncDialogRef() {
        return new Promise((resolve) => {
            requestAnimationFrame(() => {
                if (this.dialogRef)
                    resolve(this.dialogRef);
            });
        });
    }
    endTransition() {
        this.dialogRef?.executeTransition('end');
    }
    triggerOnClosedWhenTimeup(shouldTriggerOnClosed) {
        if (!shouldTriggerOnClosed)
            return;
        setTimeout(() => {
            this.triggerOnClosed();
        }, 250);
    }
    startTransition() {
        if (!this.dialogSlot?.value)
            return;
        this.createSlotRef(this.dialogSlot.value, () => {
            this.dialogRef?.executeTransition('start');
            this.attachEscapeKeyupEvent();
        });
    }
    SetOpacity(value) {
        setTimeout(() => {
            ModalSingleton.modalRef.style.setProperty('--opacity', value);
        }, 0);
    }
    openChildDialog() {
        setTimeout(() => {
            this.setModalSlot(DialogState.DIALOG_SLOT_DEFAULT);
            this.open({ slotName: this.slotName });
        }, 250);
    }
    closeDialogWhenEnd() {
        setTimeout(() => {
            this.toggleBackdrop('disabled');
            this.removeDialogLocal();
            this.setModalSlot(DialogState.DIALOG_SLOT_DEFAULT);
        }, 250);
    }
    removeDialogLocal() {
        if (this.dialogSlot?.value?.name === DialogState.LOCAL_DIALO_SLOT) {
            this.dialogRef?.remove();
        }
    }
    toggleBackdrop(status, dialogParentParam) {
        const dialogParent = this.dialogSlot?.value?.parentElement || dialogParentParam;
        if (!dialogParent)
            return;
        this.enabledBackdrop(dialogParent, status);
        this.disabledBackdrop(dialogParent, status);
    }
    enabledBackdrop(dialogParent, status) {
        if (status !== 'enabled')
            return;
        dialogParent.classList.add(DialogState.DIALOG_ENABLED);
        dialogParent.classList.remove(Modal.MODAL_DISABLED);
    }
    disabledBackdrop(dialogParent, status) {
        if (status !== 'disabled')
            return;
        dialogParent.classList.add(Modal.MODAL_DISABLED);
        dialogParent.classList.remove(DialogState.DIALOG_ENABLED);
    }
    attachEscapeKeyupEvent() {
        if (ModalSingleton.modalRef.set.disabledBackdrop || this.dialogRef?.set.disabledBackdrop)
            return;
        document.addEventListener('keydown', this.disabledBackdropByEscape);
    }
    createSlotRef(target, callback) {
        mutableElementManyTypes({
            target,
            attributes: (mutation, observer) => {
                this.setComponentRef(mutation.target);
                observer.disconnect();
                if (callback)
                    callback();
            },
        });
    }
    setComponentRef(slot) {
        this.dialogRef = slot.assignedElements()[0];
    }
    triggerOnClosed() {
        if (this.dialogRef)
            this.dialogRef.onClosed();
    }
    removeEscapeKeyupEvent() {
        document.removeEventListener('keydown', this.disabledBackdropByEscape);
    }
}
DialogState.DIALOG_SLOT_DEFAULT = 'dialog-disabled';
DialogState.DIALOG_ENABLED = 'dialog__enabled';
DialogState.LOCAL_DIALO_SLOT = 'local-dialog-slot';
//# sourceMappingURL=dialog.state.js.map