var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _PopoverState_firstUpdated, _PopoverState_closedDone;
import { debounce } from '../../../../helpers/debounceTimer';
import { Modal } from '../../modal';
import { resizeObserver } from '../../obsrevers/resize.observer';
import { ModalSingleton } from '../../singleton/modal.singleton';
import { PopoverPosition } from './PopoverPosition';
import '../../../transition/transition';
import { delay } from '../../../../helpers/delay';
export const debouceTimerPopoverResize = 200;
export class PopoverState {
    constructor() {
        _PopoverState_firstUpdated.set(this, true);
        _PopoverState_closedDone.set(this, false);
        this.unObserveResizeEvent = () => {
            this.resizeObserver.unobserve(document.body);
        };
        this.closePopover = async (e) => {
            if (__classPrivateFieldGet(this, _PopoverState_closedDone, "f"))
                return;
            if (e?.relatedTarget?.closest('c-box[slot="popover"]'))
                return;
            this.setPopoverContentAnimation('out');
            await delay(250);
            this.setIsFirstUpdate(true);
            this.unObserveResizeEvent();
            this.appendContentBackToRoot();
            this.removeMouseEvent();
            this.setPopoverClosedDone(true);
            this.onEvent('closed');
        };
    }
    async delayWhenOldPopoverExist() {
        if (ModalSingleton.modalRef.querySelector('c-box'))
            await delay(110);
    }
    // 📌w8 for fix. this is utils method can use for improve UX in future
    // public setAlertOldPopover(popover: CBox.Ref) {
    //   popover.firstElementChild?.classList.add('shake-efx');
    //   const timer = setTimeout(() => {
    //     popover.firstElementChild?.classList.remove('shake-efx');
    //     clearTimeout(timer);
    //   }, 600);
    // }
    setPopoverClosedDone(done) {
        __classPrivateFieldSet(this, _PopoverState_closedDone, done, "f");
    }
    open(popoverContent, hostRect, popoverSet, popoverHost, triggerEvent) {
        this.clearTooltip(popoverContent);
        this.setPopoverClosedDone(false);
        this.setProperties(popoverContent, hostRect, popoverSet, popoverHost);
        this.setPopoverContentAnimation('in');
        this.setPopoverContentOpacity('0');
        this.setContentTabindex();
        this.setPositionWithResizeEvent();
        this.setContentInlineBlock();
        this.setFocusOutEventListener();
        this.setPopoverContentAppendToModal();
        this.setMouseleaveEvent();
        requestAnimationFrame(() => {
            this.setPopoverContentOpacity('1');
            this.onEvent('opened', triggerEvent);
        });
        return this;
    }
    clearTooltip(popoverContent) {
        const isTooltip = popoverContent.hasAttribute('tooltip');
        if (isTooltip) {
            const tooltips = ModalSingleton.modalRef.querySelectorAll('[tooltip]');
            tooltips.forEach((ele) => {
                ele.popoverState.closePopover(null);
            });
        }
    }
    onEvent(event, triggerEvent) {
        this.popoverHost.dispatchEvent(new CustomEvent(event, {
            detail: {
                event: triggerEvent,
                state: event,
            },
        }));
    }
    setPositionWithResizeEvent() {
        this.resizeObserver = resizeObserver(document.body, (resizeEntry) => {
            if (__classPrivateFieldGet(this, _PopoverState_firstUpdated, "f")) {
                this.setPosition(resizeEntry);
                this.setIsFirstUpdate(false);
            }
            else {
                debounce(() => this.setPosition(resizeEntry), debouceTimerPopoverResize);
            }
        });
    }
    setPopoverContentAppendToModal() {
        ModalSingleton.popoverSlotRef.name = PopoverState.POPOVER_SLOT_OPEN;
        ModalSingleton.popoverSlotRef.parentElement.classList.remove(Modal.MODAL_DISABLED);
        ModalSingleton.modalRef.append(this.popoverContent);
    }
    setMouseleaveEvent() {
        if (this.popoverSet.mouseleave === 'none')
            return;
        // this.popoverContent.onmouseover = () => {
        this.popoverContent.onmouseover = () => {
            this.popoverContent.onmouseleave = this.closePopover;
        };
        this.popoverHost.onmouseleave = this.closePopover;
    }
    setProperties(popoverContent, hostRect, popoverSet, popoverHost) {
        this.popoverContent = popoverContent;
        this.hostRect = hostRect;
        this.positionType = popoverSet.position;
        this.popoverSet = popoverSet;
        this.popoverHost = popoverHost;
        this.content = popoverContent.firstElementChild;
        this.separatedPositionType = this.positionType.split('-');
    }
    setContentTabindex() {
        this.content.setAttribute('tabindex', '0');
    }
    async setPosition(resizeEntry) {
        const positionResult = await new PopoverPosition(this.positionType, this.hostRect, this.popoverContent, resizeEntry, this.separatedPositionType, this.popoverSet).getResult();
        this.setArrowpoint(positionResult);
        if (this.popoverContent.style.translate === positionResult.translate)
            return;
        this.popoverContent.style.translate = positionResult.translate;
    }
    // 📌set attribute for benefit to c-box
    setArrowpoint(positionResult) {
        if (!this.popoverSet.arrowpoint)
            return;
        const { width: hostWidth, height: hostHeight } = this.hostRect;
        const { width: contentWidth, height: contentHeight } = this.popoverContent.getBoundingClientRect();
        const { positionChecked, sideChecked } = positionResult;
        const hostCenterWidth = hostWidth / 2;
        const hostCenterHeight = hostHeight / 2;
        this.setArrowPositionTypeAttr(positionChecked);
        const arrowPosition = this.calcArrowPosition(positionChecked, sideChecked, contentWidth, contentHeight, hostCenterWidth, hostCenterHeight);
        this.setArrowTranslate(arrowPosition, positionChecked);
    }
    setArrowTranslate(arrowPosition, positionChecked) {
        const popoverStyles = this.popoverContent.style;
        const arrowPointProperty = '--popover-arrowpoint-position';
        switch (positionChecked) {
            case 'top':
            case 'bottom':
                // 📌8 is size of arrow
                return popoverStyles.setProperty(arrowPointProperty, `${arrowPosition - 8}px 0`);
            case 'left':
            case 'right':
                return popoverStyles.setProperty(arrowPointProperty, `0 ${arrowPosition - 8}px`);
            default:
                throw Error('positionChecked value is not "top" | "bottom" | "left" | "right"');
        }
    }
    setArrowPositionTypeAttr(positionChecked) {
        this.popoverContent.setAttribute(`popover-arrowpoint-position-type`, positionChecked);
    }
    calcArrowPosition(positionChecked, sideChecked, contentWidth, contentHeight, hostCenterWidth, hostCenterHeight) {
        switch (positionChecked) {
            case 'top':
            case 'bottom':
                switch (sideChecked) {
                    case 'center':
                        return Math.floor(contentWidth / 2);
                    case 'left':
                        return Math.floor(hostCenterWidth);
                    case 'right':
                        return Math.abs(Math.floor(hostCenterWidth - contentWidth));
                }
            case 'left':
            case 'right':
                switch (sideChecked) {
                    case 'center':
                        return Math.abs(Math.floor(contentHeight / 2));
                    case 'top':
                        return Math.floor(hostCenterHeight);
                    case 'bottom':
                        return Math.abs(Math.floor(hostCenterHeight - contentHeight));
                }
            default:
                throw Error('Method setArrowpoint constant arrowPosition is undefiend');
        }
    }
    setFocusOutEventListener() {
        this.popoverContent.tabIndex = 0;
        if (this.popoverSet.focusout === 'close') {
            this.popoverContent.addEventListener('focusout', this.closePopover);
        }
        requestAnimationFrame(() => {
            this.popoverContent.focus();
        });
    }
    setIsFirstUpdate(update) {
        __classPrivateFieldSet(this, _PopoverState_firstUpdated, update, "f");
    }
    removeMouseEvent() {
        if (this.popoverSet.mouseleave === 'none')
            return;
        this.popoverContent.onmouseover = null;
        this.popoverContent.onmouseleave = null;
        this.popoverHost.onmouseleave = null;
    }
    setPopoverContentAnimation(status) {
        const [positionType] = this.separatedPositionType;
        this.popoverContent.style.animation = `popover-${positionType}-${status} 0.3s ease forwards`;
    }
    appendContentBackToRoot() {
        requestAnimationFrame(() => {
            this.popoverHost.append(this.popoverContent);
        });
    }
    setContentInlineBlock() {
        const [position] = this.popoverSet.position.split('-');
        if (position === 'top' || position === 'bottom') {
            this.popoverContent.style.display = 'inline-block';
        }
        else if (position === 'left' || position === 'right') {
            this.popoverContent.style.display = 'inline-flex';
        }
    }
    setPopoverContentOpacity(opacity) {
        this.popoverContent.style.opacity = opacity;
    }
}
_PopoverState_firstUpdated = new WeakMap(), _PopoverState_closedDone = new WeakMap();
PopoverState.POPOVER_SLOT_DISABLED = 'popover-disabled';
PopoverState.POPOVER_SLOT_OPEN = 'popover';
//# sourceMappingURL=popover.state.js.map