var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { css, html } from 'lit';
import { customElement, state } from 'lit/decorators.js';
import { createRef, ref } from 'lit/directives/ref.js';
import { ComponentBase } from '../../base/component-base/component.base';
import { TransitionBuilder } from './builder/transition.builder';
import { CxTransitionName } from './types/transition.name';
let Transition = class Transition extends ComponentBase {
    constructor() {
        super(...arguments);
        this.config = {
            start: {
                name: 'scale-up',
                delay: '0',
                duration: '0.25s',
                timing: 'ease',
            },
            end: {
                name: 'scale-down',
                delay: '0',
                duration: '0.25s',
                timing: 'ease',
            },
        };
        this.transitionRef = createRef();
        this.stylesRef = createRef();
        this.FADE = 'fade';
    }
    connectedCallback() {
        super.connectedCallback();
        if (this.set)
            this.cacheConfig(this.set);
        if (this.config)
            this.exec();
    }
    render() {
        return html `
      <style ${ref(this.stylesRef)}>
        ${this.transitionCssText}
      </style>
      <div class="transition" ${ref(this.transitionRef)}>
        <slot></slot>
      </div>
    `;
    }
    willUpdate(changedProperties) {
        if (changedProperties.has('set')) {
            this.shouldForward(true);
        }
        super.willUpdate(changedProperties);
    }
    shouldForward(shouldForward) {
        if (this.set.start?.fillMode !== 'forwards' || !this.set?.fade || !shouldForward)
            return;
        this.style.visibility = 'hidden';
    }
    setTransition(when) {
        if (!this.set.start || !this.set.end)
            return;
        const cssText = new TransitionBuilder({
            transition: this.set[when],
            when,
            fade: this.set.fade,
        })
            .setTransition()
            .setFade()
            .minimize();
        this.setTransitionCSSText(cssText);
        setTimeout(() => {
            this.addFade(when);
            // 📌timeout = 0 coz fade need initial opacity:0 early
        }, 0);
        setTimeout(() => {
            this.toggleTransition(when);
            // 📌set duration here when component dosnt have fade
            if (!this.set.fade)
                this.setDuration(when);
        }, this.timeoutWhenStart(when));
        setTimeout(() => {
            this.clearTransition(when);
            this.shouldForward(when === 'end');
            // 📌after transition finish need (after 250ms) need to clear everything
        }, 250);
    }
    clearTransition(when) {
        this.clearTransitionWhenCompleted(when);
        this.clearFadeClassAfterTransitionDone();
        this.clearDuration();
        this.removeTransitionCssText();
    }
    removeTransitionCssText() {
        this.transitionCssText = '';
    }
    clearFadeClassAfterTransitionDone() {
        if (!this.set.fade)
            return;
        this.transitionRef.value?.classList.remove(this.FADE);
    }
    addFade(when) {
        if (!this.set.end || !this.set.start || !this.set.fade)
            return;
        this.transitionRef.value?.classList.add(this.FADE);
        this.setDuration(when);
    }
    setDuration(when) {
        // 📌use setAttribute improve performance and must be aware that there is no set style other than this below
        this.setAttribute('style', `--duration:${this.set[when]?.duration};--delay:${this.set[when]?.delay};--timing:${this.set[when]?.timing}`);
    }
    clearDuration() {
        this.setAttribute('style', '');
    }
    timeoutWhenStart(when) {
        // 📌if fade = 'none' && when = 'start' it's mean child of cx-transition always appear it does not need 250ms for delay
        // 📌fade != 'none' mean transition has fade. fade need to init opacity = 0 with 0ms *before content appear*
        if (!this.set.fade || when)
            return 0;
        // 📌250 is duration time of each transitions
        return 250;
    }
    // 📌need to clear class when completed coz, bug occur
    clearTransitionWhenCompleted(when) {
        if (!this.set[when])
            return;
        this.transitionRef.value?.classList.remove(this.set[when].name);
    }
    setTransitionCSSText(cssText) {
        this.transitionCssText = cssText;
    }
    toggleTransition(when) {
        if (!this.set.end || !this.set.start)
            return;
        if (when === 'start') {
            this.transitionRef.value?.classList.remove(this.set.end.name);
            this.transitionRef.value?.classList.add(this.set.start.name);
        }
        else if (when === 'end') {
            this.transitionRef.value?.classList.remove(this.set.start.name);
            this.transitionRef.value?.classList.add(this.set.end.name);
        }
    }
};
Transition.styles = css `
    :host {
      display: inline-block;
    }
    .transition {
      transition-duration: var(--duration);
      transition-delay: var(--delay);
      transition-timing-function: var(--timing);
      transition-property: all;
    }
  `;
__decorate([
    state(),
    __metadata("design:type", Object)
], Transition.prototype, "transitionCssText", void 0);
Transition = __decorate([
    customElement(CxTransitionName)
], Transition);
export { Transition };
//# sourceMappingURL=transition.js.map